package com.taco.swinger.text2gui.tools;

import java.io.IOException;
import java.io.InputStream;

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

import com.taco.util.AccessibleResourceBundle;
import com.taco.util.ResourceBundleFactory;
import com.taco.util.MultiLinePropertyResourceBundle;

/** A resource bundle factory that can load a bundle based on a string. It 
 *  needs to be constructed with the string first. To get a bundle based on
 *  the string, use "&lt;STRING&gt;" as the bundle name.
 */  
public class StringResourceBundleFactory extends ResourceBundleFactory {
    public StringResourceBundleFactory(CharSequence cs) {
        if (cs == null) {
            throw new NullPointerException();
        }      
        _cs = cs;                       
    }
    
    /** An input stream based on a character sequence that just truncates the
     *  upper 8 bits.
     */
    private class CharSequenceInputStream extends InputStream {
        public CharSequenceInputStream() { }
            
        public int read() {
            if (_readPosition >= _cs.length()) {
                return -1;
            }
            
            return (_cs.charAt(_readPosition++) & 0xff);
        }
        
        private int _readPosition = 0;
    }
    
    /** If <code>bundleName</code> is "&lt;STRING;&gt;", return a bundle based
     *  on the character sequence passed to the constructor. Otherwise, use the
     *  superclass's implementation.
     */
    protected AccessibleResourceBundle _loadOrphanBundle(
      final String bundleName, final Locale defaultLocale, 
      final ClassLoader loader) throws MissingResourceException {
        if (bundleName == null) {
            throw new NullPointerException();
        }
          
        if ("<STRING>".equals(bundleName)) {
            InputStream is = new CharSequenceInputStream();
            
            try {
                return new MultiLinePropertyResourceBundle(is);
            } catch (IOException ioe) {
                throw new MissingResourceException("Can't load from string",
                 "<STRING>", null);
            }        
        } else {
            return super._loadOrphanBundle(bundleName, defaultLocale, loader);               
        }
    }    
    
    protected final CharSequence _cs;
}

